//
// Deferred Shading pass 2, using only RGBA8 textures and depth map
//
#extension GL_ARB_texture_rectangle: enable

varying	vec3 pos;
uniform vec3 lightPos;
uniform sampler2DRect	depthMap;
uniform sampler2DRect	normalMap;
uniform sampler2DRect	colorMap;

#define	MAX_DIST	1.5

float	atten ( float d )								// compute distance attenuation factor, 0 -> 1, MAX_DIST -> 0
{
	const	float	offs0 = 1.0;
	const	float	offs1 = 1.0 / (1.0 + MAX_DIST);
	const	float	scale = 1.0 / (offs0 - offs1);
	
	return scale * ( 1.0 / (1.0 + d) - offs1 );
}

void main (void)
{
	const float zFar  = 100.0;
	const float zNear = 0.01;

	float	zb = texture2DRect ( depthMap, gl_FragCoord.xy ).x;
	float	z  = zFar*zNear/(zb * (zFar - zNear) - zFar);
	vec3	n  = 2.0*(texture2DRect ( normalMap, gl_FragCoord.xy ).xyz - vec3 ( 0.5) );
	vec4	c  = texture2DRect ( colorMap,  gl_FragCoord.xy );
	vec3	pp = pos * z / pos.z;
	vec3	ll = lightPos - pp;
	float	d  = length ( ll );
	
	if ( d > MAX_DIST )									// too far away
		discard;
		
	float	k  = atten ( d );
	vec3	l  = ll / d;
	vec3	v  = normalize ( -pp );
	vec3	h  = normalize ( l + v );
	float	diff = max ( 0.0, dot ( l, n ) );
	float	spec = pow ( max ( 0.0, dot ( h, n ) ), 40.0 );
	
	gl_FragColor = k * vec4 ( diff * c.rgb + vec3 ( spec ), 1.0 );
}
